/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.bwsin.konfig.aktuell.AufteilungspunktAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuellesObjekt;
import de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt;
import de.bsvrz.kex.bwsin.konfig.kb.SINStrassenSegment;
import de.bsvrz.kex.bwsin.vew.SINProperties;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abweichungsliste mit der Zusammenfassung der Importoperation.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: Abweichungsliste.java 20866 2009-12-22 15:39:06Z gieseler $
 * 
 */
public class Abweichungsliste {

	/**
	 * Interne Repr&auml;sentation eines gefundenen Netz-Referenzproblemes.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * @version $Id: Abweichungsliste.java 20866 2009-12-22 15:39:06Z gieseler $
	 * 
	 */
	private class ReferenzProblem {
		/**
		 * Das referenzierende Objekt.
		 */
		SystemObject refrenzierendesObjekt = null;

		/**
		 * Das referenzierte Objekt.
		 */
		ConfigurationObject refrenziertesObjekt = null;

		/**
		 * Die Art des Referenzierungsproblems.
		 */
		ReferenzProblemArt problemArt = null;

		/**
		 * Konstruktor.
		 * 
		 * @param refrenzierendesobjekt
		 *            das referenzierende Objekt
		 * @param refrenziertesobjekt
		 *            das referenzierte Objekt
		 * @param problemart
		 *            die Art des Problems
		 */
		public ReferenzProblem(SystemObject refrenzierendesobjekt,
				ConfigurationObject refrenziertesobjekt,
				ReferenzProblemArt problemart) {
			refrenzierendesObjekt = refrenzierendesobjekt;
			refrenziertesObjekt = refrenziertesobjekt;
			problemArt = problemart;
		}
	}

	/**
	 * Art des Referezierungsproblems.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * @version $Id: Abweichungsliste.java 20866 2009-12-22 15:39:06Z gieseler $
	 * 
	 */
	private static final class ReferenzProblemArt {

		/** referenziertes Objekt existiert nicht mehr. */
		public static final ReferenzProblemArt REFERENZ_EXISTIERT_NICHT = new ReferenzProblemArt(
				"nicht mehr existiert"); //$NON-NLS-1$

		/** Offset ung&uuml;ltig. */
		public static final ReferenzProblemArt OFFSET_ZU_GROSS = new ReferenzProblemArt(
				"eine kleinere Lnge als der angegebene Offset besitzt"); //$NON-NLS-1$

		/** Art des Problems. */
		private final String text;

		/**
		 * Nicht &ouml;ffentlicher Konstruktor der zum Erzeugen der
		 * vordefinierten Zust&auml;nde benutzt wird.
		 * 
		 * @param text
		 *            textuelle Beschreibung des Problems
		 */
		private ReferenzProblemArt(final String text) {
			this.text = text;
		}

		/**
		 * Liefert eine textuelle Beschreibung dieses Verbindungszustands
		 * zur&uuml;ck. Das genaue Format ist nicht festgelegt und kann sich
		 * &auml;ndern.
		 * 
		 * @return Beschreibung dieses Zustands.
		 */
		@Override
		public String toString() {
			return text;
		}

	}

	/**
	 * Indizes f&uuml;r die Zugriffe auf die Elemente der Netzreferenzen
	 */
	/**
	 * Index f&uuml;r den Zugriff auf den Objekttyp.
	 */
	public static final int TYP = 0;

	/**
	 * Index f&uuml;r den Zugriff auf die Attributgruppe.
	 */
	public static final int ATG = 1;

	/**
	 * Index f&uuml;r den Zugriff auf den Aspekt.
	 */
	public static final int ASP = 2;

	/**
	 * Index f&uuml;r den Zugriff auf den Attributnamen.
	 */
	public static final int ATT = 3;

	/**
	 * DebugLogger f&uuml;r Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Die Liste der neu erzeugten Objekte.
	 */
	private List<String> neueObjekte = new ArrayList<String>();

	/**
	 * Die Liste der gel&ouml;schten Objekte.
	 */
	private List<String> geloeschteObjekte = new ArrayList<String>();

	/**
	 * Die Liste der ge&auml;nderten Objekte.
	 */
	private final List<String> geaenderteObjekte = new ArrayList<String>();

	/**
	 * Modul AktuelleKonfiguration.
	 */
	private final SINAktuelleKonfiguration aktuelleKonfiguration;

	/**
	 * Die durch den Import erzeugten Konfigurationsobjekte.
	 */
	private final Map<String, KonfigurationsObjekt> neueKonfiguration;

	/**
	 * Die Referenzprobleme.
	 */
	private final List<ReferenzProblem> referenzProbleme = new ArrayList<ReferenzProblem>();

	/**
	 * Die durch den Import erzeugte Datei.
	 */
	private final File kbDatei;

	/**
	 * Erzeugt eine Instanz.
	 * 
	 * @param aktuelleKonfiguration
	 *            Instanz zum Zugriff auf die aktuelle Konfiguration
	 * @param neueKonfiguration
	 *            Die durch den Import erzeugten Konfigurationsobjekte
	 * @param kbfile
	 *            Datei, in der der neue KB gespeichert wurde
	 */
	public Abweichungsliste(SINAktuelleKonfiguration aktuelleKonfiguration,
			Map<String, KonfigurationsObjekt> neueKonfiguration, File kbfile) {
		this.aktuelleKonfiguration = aktuelleKonfiguration;
		this.neueKonfiguration = neueKonfiguration;
		this.kbDatei = kbfile;
	}

	/**
	 * F&uuml;hrt das Erstellen der Abweichungsliste aus.
	 */
	public void erstellen() {

		debug.info("Abweichungsliste wird erstellt"); //$NON-NLS-1$

		// Listen vergleichen
		neueObjekte = new ArrayList<String>(neueKonfiguration.keySet());
		List<String> aktuelleobjekte = aktuelleKonfiguration
				.getAktuelleObjekteTeilmodellVerkehr();

		List<String> ueberpruefen = new ArrayList<String>(neueObjekte);
		
		// berprfe alle neuen Objekte, die auch in der aktuellen Konfiguration
		// enthalten sind
		ueberpruefen.retainAll(aktuelleobjekte);
		
		List<String> dup = new ArrayList<String>(neueObjekte);
		
		neueObjekte.removeAll(aktuelleobjekte);

		aktuelleobjekte.removeAll(dup);

		geloeschteObjekte = aktuelleobjekte;

		ueberpruefeGleichheit(ueberpruefen);

		ueberpruefeNetzreferenzen();
	}

	/**
	 * Gibt die Abweichungsliste als Text in eine Datei aus.
	 * 
	 * @param dateiname
	 *            zu schreibende Datei
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	public void inDatei(String dateiname) throws ImportException {
		PrintStream ps;
		try {
			ps = new PrintStream(new File(dateiname));
		} catch (FileNotFoundException e) {
			throw new ImportException(
					"Die Protokolldatei kann nicht erstellt werden (" + //$NON-NLS-1$
							e.getMessage() + ")"); //$NON-NLS-1$
		}

		debug.info("Die Abweichungsliste wird in die Datei '" + //$NON-NLS-1$
				dateiname + "' ausgegeben"); //$NON-NLS-1$

		this.print(ps);

		ps.close();
	}

	/**
	 * Ausgabe der Abweichungsliste in textueller Form auf einen Stream.
	 * 
	 * @param ps
	 *            Ausgabestream
	 * 
	 */
	@SuppressWarnings("nls")
	public void print(PrintStream ps) {
		Calendar cal = Calendar.getInstance();
		DateFormat df = new SimpleDateFormat();

		String[] myargs = { SINProperties.PROP_KEY_WITH_GUI,
				SINProperties.PROP_KEY_QUELLDATENBASIS,
				SINProperties.PROP_KEY_LCL, SINProperties.PROP_KEY_KBPFAD,
				SINProperties.PROP_KEY_KV, SINProperties.PROP_KEY_KBPID,
				SINProperties.PROP_KEY_KBNAME, SINProperties.PROP_KEY_KBINFO,
				SINProperties.PROP_KEY_ABW,
				SINProperties.PROP_KEY_ABW_PROTOKOLLDATEI,
				SINProperties.PROP_KEY_REFERENZEN };

		ps
				.println("=====================================================================================");
		ps.println("Abweichungsliste Statischer Import Netz vom "
				+ df.format(cal.getTime()));
		ps
				.println("=====================================================================================");
		ps
				.println("Die Applikation wurde mit folgenden Aufrufparametern gestartet:");
		ps
				.println("-------------------------------------------------------------------------------------");
		for (int i = 0; i < myargs.length; i++) {
			ps.format("%-31s = ", myargs[i]);
			ps.println(SINProperties.getInstance().getProperty(myargs[i]));
		}
		ps
				.println("-------------------------------------------------------------------------------------");
		ps.println("erzeugte Datei Konfigurationsbereich: "
				+ kbDatei.getAbsolutePath());
		ps
				.println("-------------------------------------------------------------------------------------");
		ps.println("Abweichungen - neue Objekte");
		ps
				.println("-------------------------------------------------------------------------------------");
		for (String pid : neueObjekte) {
			KonfigurationsObjekt ko = neueKonfiguration.get(pid);
			String typename = ko.getDavtype().getName();
			ps.println("Objekt vom Typ " + typename + " mit der PID "
					+ ko.getPid() + ", Name " + ko.getName());
		}
		ps
				.println("-------------------------------------------------------------------------------------");
		ps.println("Abweichungen - gelschte Objekte");
		ps
				.println("-------------------------------------------------------------------------------------");
		for (String pid : geloeschteObjekte) {
			SystemObject so = aktuelleKonfiguration.getObjekt(pid)
					.getDavObjekt();
			String typename = so.getType().getName();
			ps.println("Objekt vom Typ " + typename + " mit der PID " + pid
					+ ", Name " + so.getName());
		}
		ps
				.println("-------------------------------------------------------------------------------------");
		ps.println("Abweichungen - genderte Objekte");
		ps
				.println("-------------------------------------------------------------------------------------");
		for (String pid : geaenderteObjekte) {
			KonfigurationsObjekt ko = neueKonfiguration.get(pid);
			String typename = ko.getDavtype().getName();
			ps.println("Objekt vom Typ " + typename + " mit der PID " + pid
					+ ", Name " + ko.getName()
					+ " hat einen genderten Polygonzug");
		}
		ps
				.println("-------------------------------------------------------------------------------------");
		ps.println("Problematische Referenzen");
		ps
				.println("-------------------------------------------------------------------------------------");

		for (ReferenzProblem rp : referenzProbleme) {
			ps.println("Referenz vom Objekt "
					+ rp.refrenzierendesObjekt.getType().getName()
					+ " mit der PID " + rp.refrenzierendesObjekt.getPid()
					+ " (" + rp.refrenzierendesObjekt.getName() + ") "
					+ "auf das Objekt "
					+ rp.refrenziertesObjekt.getType().getName()
					+ " mit der PID " + rp.refrenziertesObjekt.getPid() + " ("
					+ rp.refrenziertesObjekt.getName() + ") "
					+ "nicht mglich, da das "
					+ rp.refrenziertesObjekt.getType().getName() + " "
					+ rp.problemArt);
		}
		ps
				.println("-------------------------------------------------------------------------------------");
		ps
				.println("Nicht verwendete Aufteilungspunkte der aktuellen Konfiguration mit Koordinaten");
		ps
				.println("-------------------------------------------------------------------------------------");
		if (aktuelleKonfiguration
				.getNichtVerarbeiteteAufteilungspunkteKoordinaten() != null) {
			for (AufteilungspunktAktuelleKonfiguration ap : aktuelleKonfiguration
					.getNichtVerarbeiteteAufteilungspunkteKoordinaten()) {
				ps.println(ap.getSystemObjekt().getPid() + "("
						+ ap.getSystemObjekt().getName() + ")");
			}
		}

		ps
				.println("-------------------------------------------------------------------------------------");
		ps
				.println("Nicht verwendete Aufteilungspunkte der aktuellen Konfiguration mit Linienreferenz");
		ps
				.println("-------------------------------------------------------------------------------------");
		if (aktuelleKonfiguration
				.getNichtVerarbeiteteAufteilungspunkteLinienreferenz() != null) {
			for (AufteilungspunktAktuelleKonfiguration ap : aktuelleKonfiguration
					.getNichtVerarbeiteteAufteilungspunkteLinienreferenz()) {
				ps.println(ap.getSystemObjekt().getPid() + "("
						+ ap.getSystemObjekt().getName() + ")");
			}
		}

		ps
				.println("-------------------------------------------------------------------------------------");

	}

	/**
	 * &Uuml,berpr&uuml;ft eine Liste von Objekten auf Gleichheit mit den
	 * Objekten gleicher PID der aktuellen Konfiguration. Als gleich werden
	 * Objekte angesehen, deren Koordinaten sich nicht mehr als eine definierte
	 * Abweichung unterscheiden. Die max. Abweichung ist durch die Property
	 * SINProperties.PROP_KEY_MAX_ABWEICHUNG festgelegt. Wird eine Abweichung
	 * festgestellt, wird das entsprechende Objekt in der Liste der
	 * ge&auml;derten Objekte gespeichert.
	 * 
	 * @param objekte
	 *            die Liste der zu &uuml;berpr&uuml;fenden Objekte
	 */
	private void ueberpruefeGleichheit(List<String> objekte) {
		double maxabweichung = SINProperties.getInstance()
				.getMaximaleAbweichungGrad();

		debug
				.info("Die Konfigurationsobjekte werden auf bereinstimmung geprft"); //$NON-NLS-1$

		for (String pid : objekte) {
			KonfigurationsObjekt ko = neueKonfiguration.get(pid);
			if (ko instanceof SINStrassenSegment) {
				WGS84Polygon neuesPolygon = new WGS84Polygon(ko
						.getKoordinaten());
				WGS84Polygon aktuellesPolygon = aktuelleKonfiguration
						.getObjekt(pid).getPolygon();

				if (!neuesPolygon.istIdentisch(aktuellesPolygon, maxabweichung)) {
					geaenderteObjekte.add(pid);
				}
			}
		}
	}

	/**
	 * &Uuml,berpr&uuml;ft eine Netzreferenz der aktuellen Konfiguration. Alle
	 * festgestellten Referenzprobleme der aktuellen Konfiguration
	 * bez&uuml;glich des neu erzeugten Konfigurationsbereiches werden in einer
	 * Liste gespeichert. In Erweiterung der Afo werden zus&auml;tzlich die
	 * Offsets auf G&uuml;ltigkeit gepr&uuml;ft.
	 * 
	 * @param netzreferenz
	 *            eine Netzreferenz
	 * @throws ImportException
	 *             bei Ausnahmen
	 */
	private void ueberpruefeNetzreferenz(Object[] netzreferenz)
			throws ImportException {
		if (netzreferenz.length < ATT) {
			throw new ImportException("Fehlende Angaben bei Netzreferenz"); //$NON-NLS-1$
		}

		// einlesen aller Objekte des Typs
		List<SINAktuellesObjekt> elemente = aktuelleKonfiguration
				.getObjektListeGlobal((String) netzreferenz[TYP]);

		AttributeGroup atg = aktuelleKonfiguration
				.getAttributeGroup((String) netzreferenz[ATG]);

		for (SINAktuellesObjekt ao : elemente) {
			SystemObject so = ao.getDavObjekt();
			Data d = so.getConfigurationData(atg);
			if (d != null) {
				ConfigurationObject ref = (ConfigurationObject) d
						.getReferenceValue((String) netzreferenz[ATT])
						.getSystemObject();

				if (ref != null) {
					// die Referenz muss von Typ "typ.straenSegment" sein
					if (ref.isOfType("typ.straenSegment")) { //$NON-NLS-1$
						if (neueKonfiguration.get(ref.getPid()) == null) {
							debug
									.finest("Die Objektreferenz: " + so.getNameOrPidOrId() + " --> " + //$NON-NLS-1$ //$NON-NLS-2$
											ref.getNameOrPidOrId()
											+ " ist nicht mehr gltig"); //$NON-NLS-1$
							referenzProbleme
									.add(new ReferenzProblem(
											so,
											ref,
											ReferenzProblemArt.REFERENZ_EXISTIERT_NICHT));
						} else {
							// berprfe die Lnge
							if (atg.getAttribute("Offset") != null) { //$NON-NLS-1$
								float offset = d
										.getScaledValue("Offset").floatValue(); //$NON-NLS-1$

								if (offset > neueKonfiguration
										.get(ref.getPid()).getLaenge()) {
									referenzProbleme
											.add(new ReferenzProblem(
													so,
													ref,
													ReferenzProblemArt.OFFSET_ZU_GROSS));
								}
							}
						}
					} else {
						debug
								.error("nicht untersttzter Referenztyp zur berprfung: " + ref.getType().getName()); //$NON-NLS-1$
					}
				}
			}
		}
	}

	/**
	 * &Uuml,berpr&uuml;ft die Netzreferenzen der aktuellen Konfiguration. Die
	 * zu pr&uuml;fenden Referenzen sind &uuml;ber eine java-Datei konfiguriert,
	 * die mit dem Aufrufparameter netzReferenzen &uuml;bergeben werden kann und
	 * in der Property SINProperties.PROP_KEY_REFERENZEN gespeichert ist. In
	 * dieser Datei sind die zu pr&uuml;fenden Objekttypen, Attributgruppen und
	 * Attribute in Form einer Tabelle definiert.
	 */
	private void ueberpruefeNetzreferenzen() {
		Class<?> netzreferenzenclass;
		String netzreferenzenclassname = SINProperties.getInstance()
				.getProperty(SINProperties.PROP_KEY_REFERENZEN);

		debug.info("Die Netzreferenzen werden geprft."); //$NON-NLS-1$

		try {
			netzreferenzenclass = Class.forName(netzreferenzenclassname);
			Method getmethode = netzreferenzenclass.getMethod(
					"getNetzReferenzen", new Class[] {}); //$NON-NLS-1$
			Object[][] netzreferenzen = (Object[][]) getmethode.invoke(null,
					new Object[] {});

			for (int i = 0; i < netzreferenzen.length; ++i) {
				try {
					ueberpruefeNetzreferenz(netzreferenzen[i]);
				} catch (ImportException e) {
					debug.error(e.getMessage());
				}
			}
		} catch (ClassNotFoundException e) {
			debug
					.error("Die Klasse zur berprfung der Netzreferenzen konnte nicht geladen werden: " //$NON-NLS-1$
							+ netzreferenzenclassname);
			e.printStackTrace();
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			debug.error("Methode 'getNetzReferenzen()' in der Klasse '" //$NON-NLS-1$
					+ netzreferenzenclassname + "' existiert nicht."); //$NON-NLS-1$
			e.printStackTrace();
			throw new RuntimeException(e);

		} catch (IllegalAccessException e) {
			debug
					.error("Fehler beim Zugriff auf die Methode 'getNetzReferenzen()' in der Klasse '" //$NON-NLS-1$
							+ netzreferenzenclassname + "'."); //$NON-NLS-1$ 
			e.printStackTrace();
			throw new RuntimeException(e);

		} catch (InvocationTargetException e) {
			debug.error(e.getMessage());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

}
