/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import org.geotools.feature.Feature;

/**
 * Klasse zur Repr&auml;sentation einer ASB-Stationierung.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AsbStationierung.java 21246 2010-01-22 14:39:03Z gieseler $
 * 
 */
public class AsbStationierung implements Cloneable {

	/**
	 * Das Feature des zuge&ouml;hrigen Abschnittes/Astes.
	 */
	protected Feature aoaFeature = null;

	/**
	 * Offset (Abstand des ASB-Stationierungs-Punktes vom Anfang des
	 * Linienobjekts).
	 */
	protected int offset = -1; // nicht ermittelbar

	/**
	 * Anfangsknoten des Teilabschnittes oder Astes.
	 */
	protected String anfangsKnoten = ""; //$NON-NLS-1$

	/**
	 * Endknoten des Teilabschnittes oder Astes.
	 */
	protected String endKnoten = ""; //$NON-NLS-1$

	/**
	 * Anfangsstationierung. Stationierungsangabe in Metern relativ zum
	 * Abschnitt oder Ast.
	 */
	protected int stationierungAnfang = -1; // nicht ermittelbar

	/**
	 * Endstationierung. Stationierungsangabe in Metern relativ zum Abschnitt
	 * oder Ast.
	 */
	protected int stationierungEnde = -1; // nicht ermittelbar

	/**
	 * Verkehrsrichtung.
	 */
	protected Verkehrsrichtung verkehrsRichtung = Verkehrsrichtung.RICHTUNG_UNBEKANNT;

	/**
	 * Standard-Konstruktor.
	 */
	protected AsbStationierung() {
		/**
		 * 
		 */
	}

	/**
	 * Erzeugt ein ASB-Stationierungsobjekt.
	 * 
	 * @param aoa
	 *            Feature des Abschnittes oder Astes
	 * @param offset
	 *            Abstand des ASB-Stationierungs-Punktes vom Anfang des
	 *            Linienobjekts. (noch unklar!)
	 * @param anfangsknoten
	 *            Anfangsknoten des Teilabschnittes oder Astes.
	 * @param endknoten
	 *            Endknoten des Teilabschnittes oder Astes.
	 * @param stationierunganfang
	 *            Anfangsstationierung. Stationierungsangabe in Metern relativ
	 *            zum Abschnitt oder Ast.
	 * @param stationierungende
	 *            Endstationierung. Stationierungsangabe in Metern relativ zum
	 *            Abschnitt oder Ast.
	 */
	public AsbStationierung(Feature aoa, int offset, String anfangsknoten,
			String endknoten, int stationierunganfang, int stationierungende) {
		this.aoaFeature = aoa;
		this.offset = offset;
		this.anfangsKnoten = anfangsknoten;
		this.endKnoten = endknoten;
		this.stationierungAnfang = stationierunganfang;
		this.stationierungEnde = stationierungende;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public AsbStationierung clone() {
		AsbStationierung neu = new AsbStationierung(aoaFeature, offset,
				anfangsKnoten, endKnoten, stationierungAnfang,
				stationierungEnde);
		neu.setVerkehrsRichtung(verkehrsRichtung);

		return neu;
	}

	/**
	 * Liefert den Anfangsknoten der ASB-Stationierung.
	 * 
	 * @return Anfangsknoten
	 */
	public String getAnfangsKnoten() {
		return anfangsKnoten;
	}

	/**
	 * Gibt die ID des Abschnitts/Astes zur&uuml;ck.
	 * 
	 * @return ID Abschnitt oder Ast
	 */
	public String getAOAId() {
		return aoaFeature.getID();
	}

	/**
	 * Liefert den Endknoten der ASB-Stationierung.
	 * 
	 * @return Endknoten
	 */
	public String getEndKnoten() {
		return endKnoten;
	}

	/**
	 * Liefert den Offset der ASB-Stationierung (Abstand des
	 * ASB-Stationierungs-Punktes vom Anfang des Linienobjekts. (noch unklar!)).
	 * 
	 * @return Offset
	 */
	public int getOffset() {
		return offset;
	}

	/**
	 * Gibt die Anfangsstationierung zur&uuml;ck.
	 * 
	 * @return Anfangsstationierung
	 */
	public int getStationierungAnfang() {
		return stationierungAnfang;
	}

	/**
	 * Gibt die Endstationierung zur&uuml;ck.
	 * 
	 * @return Endstationierung
	 */
	public int getStationierungEnde() {
		return stationierungEnde;
	}

	/**
	 * Gibt die Verkehrsrichtung zur&uuml;ck.
	 * 
	 * @return Verkehrsrichtung
	 */
	public Verkehrsrichtung getVerkehrsRichtung() {
		return verkehrsRichtung;
	}

	/**
	 * Setzt die Verkehrsrichtung.
	 * 
	 * @param verkehrsRichtung
	 *            Verkehrsrichtung
	 */
	protected void setVerkehrsRichtung(Verkehrsrichtung verkehrsRichtung) {
		this.verkehrsRichtung = verkehrsRichtung;
	}
}
