/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.List;

import de.bsvrz.kex.bwsin.konfig.lcl.LCL;

/**
 * Klasse zur Repr&auml;sentation von Locationen.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: Location.java 10426 2008-07-10 12:09:28Z gieseler $
 * 
 */
public class Location {

	/**
	 * Klasse zur Repr&auml;sentation der Locationrichtung.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * @version $Id: Location.java 10426 2008-07-10 12:09:28Z gieseler $
	 * 
	 */
	private class LocationRichtung {
		/**
		 * Kennung f&uuml;r die Locationrichtung.
		 */
		String kennung = ""; //$NON-NLS-1$

		/**
		 * Konstruktor aus Richtungskennung.
		 * 
		 * @param kennung
		 *            Kennung f&uuml;r die Locationrichtung
		 */
		public LocationRichtung(String kennung) {
			this.kennung = kennung;
		}

		/**
		 * {@inheritDoc}
		 * 
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return kennung;
		}
	}

	/**
	 * Konstante f&uuml;r positive Locationrichtung.
	 */
	public static final LocationRichtung LOCATION_RICHTUNG_POSITIV = (new Location()).new LocationRichtung(
			"p"); //$NON-NLS-1$

	/**
	 * Konstante f&uuml;r negative Locationrichtung.
	 */
	public static final LocationRichtung LOCATION_RICHTUNG_NEGATIV = (new Location()).new LocationRichtung(
			"n"); //$NON-NLS-1$

	/**
	 * Konstante f&uuml;r unbekannten Locationcode.
	 */
	public static final String LOCATION_CODE_UNBEKANNT = "00000"; //$NON-NLS-1$

	/**
	 * Konstante f&uuml;r unbekannte Location.
	 */
	public static final Location LOCATION_UNBEKANNT = new Location(
			LOCATION_CODE_UNBEKANNT, LOCATION_RICHTUNG_POSITIV);

	/**
	 * Vergleicht zwei Location-Listen.
	 * 
	 * @param liste1
	 *            erste Liste
	 * @param liste2
	 *            zweite Liste
	 * @return Das Ergebnis ist true wenn die beiden Listen aus den gleichen
	 *         Locationen bestehen.
	 */
	public static boolean gleicheLocationen(List<Location> liste1,
			List<Location> liste2) {
		int match = 0;

		if (liste1.size() != liste2.size()) {
			return false;
		}

		for (Location loc1 : liste1) {
			for (Location loc2 : liste2) {
				if (loc2.equals(loc1)) {
					match++;
					break;
				}
			}
		}

		return match == liste1.size();
	}

	/**
	 * Locationcode.
	 */
	private String locationCode = LOCATION_CODE_UNBEKANNT;

	/**
	 * Locationrichtung.
	 */
	private LocationRichtung locationRichtung = null;

	/**
	 * privater Konstruktor.
	 */
	private Location() {
		/**
		 * 
		 */
	}

	/**
	 * Konstruktor aus Locationangabe mit Richtung und Locationcode der Form:
	 * r.NNNNN mit r: Richtung p positiv n negativ NNNNN: Locationcode.
	 * 
	 * @param locationstring
	 *            Locationsangabe
	 */
	public Location(String locationstring) {
		locationCode = locationstring
				.substring(locationstring.indexOf('.') + 1);
		if (locationstring.substring(0, locationstring.indexOf('.'))
				.equals("p")) {
			locationRichtung = LOCATION_RICHTUNG_POSITIV;
		} else if (locationstring.substring(0, locationstring.indexOf('.'))
				.equals("n")) {
			locationRichtung = LOCATION_RICHTUNG_NEGATIV;
		} else {
			throw new IllegalArgumentException(
					"Die Location-Richtung kann nicht bestimmt werden"); //$NON-NLS-1$
		}
	}

	/**
	 * Konstruktor aus Locationcode und Richtung.
	 * 
	 * @param locationcode
	 *            Locationcode
	 * @param locationrichtung
	 *            Locationrichtung
	 */
	public Location(String locationcode, LocationRichtung locationrichtung) {
		locationCode = locationcode;
		locationRichtung = locationrichtung;
	}

	/**
	 * Bestimmt die Vorg&auml;ngerlocation der Location auf der Basis der LCL.
	 * 
	 * @return {@link Location} die Vorg&auml;ngerlocation
	 * @throws ImportException
	 *             wenn kein oder mehrere Vorg&auml;nger gefunden wurden.
	 */
	public Location bestimmeVorgaenger() throws ImportException {
		String spalte = getLocationRichtung() == Location.LOCATION_RICHTUNG_POSITIV ? LCL.SPALTE_NACHFOLGER
				: LCL.SPALTE_VORGAENGER;
		List<String> liste = LCL.getInstance().getLocationCodes(spalte,
				getLocationCode());

		if (liste.size() == 0) {
			throw new ImportException("In der LCL wurde kein Vorgnger zu '" //$NON-NLS-1$
					+ this.locationCode + "' gefunden."); //$NON-NLS-1$
		}

		if (liste.size() > 1) {
			throw new ImportException(
					"In der LCL wurden mehrere Vorgnger zu '" //$NON-NLS-1$
							+ this.locationCode + "' gefunden."); //$NON-NLS-1$
		}

		Location loc = new Location(liste.get(0), this.locationRichtung);
		if (loc.istDefiniert()) {
			return loc;
		}
		return loc.bestimmeVorgaenger();
	}

	/**
	 * Vergleicht die Location mit einer anderen.
	 * 
	 * @param loc
	 *            {@link Location}
	 * @return Das Ergebnis ist true wenn sowohl der Locationcode als auch die
	 *         Richtung &uuml;bereinstimmen.
	 */
	public boolean equals(Location loc) {
		return this.locationCode.equals(loc.locationCode)
				&& this.locationRichtung == loc.locationRichtung;
	}

	/**
	 * Gibt den Locationcode der Location zur&uuml;ck.
	 * 
	 * @return Locationcode
	 */
	public String getLocationCode() {
		return locationCode;
	}

	/**
	 * Gibt die Richtung der Location zur&uuml;ck.
	 * 
	 * @return Locationrichtung
	 */
	public LocationRichtung getLocationRichtung() {
		return locationRichtung;
	}

	/**
	 * Test ob die Location in einer Richtung definiert ist.
	 * 
	 * @return {@link Location} die Vorg&auml;ngerlocation
	 */
	public boolean istDefiniert() {
		String spalte = locationRichtung == Location.LOCATION_RICHTUNG_POSITIV ? LCL.PRESENT_POSITIVE
				: LCL.PRESENT_NEGATIVE;
		String present = LCL.getInstance().getLocationCodeWert(locationCode,
				spalte);

		if (present != null) {
			return Integer.parseInt(present) == 1;
		}

		return false;
	}

	/**
	 * Setzt den Locationcode der Location.
	 * 
	 * @param locationcode
	 *            Locationcode
	 */
	public void setLocationCode(String locationcode) {
		this.locationCode = locationcode;
	}

	/**
	 * Setzt die Richtung der Location.
	 * 
	 * @param locationrichtung
	 *            Locationrichtung
	 */
	public void setLocationRichtung(LocationRichtung locationrichtung) {
		this.locationRichtung = locationrichtung;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "" + locationRichtung + "." + locationCode;
	}
}
