/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;

/**
 * Route als geordnete Liste von Stra&szlig;enabschnitten.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: Route.java 10078 2008-07-01 18:33:07Z gieseler $
 * 
 */
public class Route extends LinkedList<StrassenAbschnitt> {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Konstruktor.
	 */
	public Route() {
		super();
	}

	/**
	 * Konstruktor.
	 * 
	 * @param route
	 *            Route
	 */
	public Route(Route route) {
		super(route);
	}

	/**
	 * Bestimmt den abschlie&szlig;enden Abschnitt der Route. Falls die Route
	 * nicht mit einem &auml;u&szlig;eren Stra&szlig;enelement endet, wird
	 * versucht, ein am Endpunkt der Route ankommendes &auml;u&szlig;eren
	 * Stra&szlig;enelement zu finden.
	 * 
	 * @param netz
	 *            Importnetz
	 * @return &auml;u&szlig;eren Stra&szlig;enelement am Ende der Route oder
	 *         null wenn nichts gefunden
	 */
	StrassenAbschnitt bestimmeEndabschnitt(final ImportNetz netz) {
		StrassenAbschnitt foundsa = null;
		if (getLast().istASS()) {
			foundsa = getLast();
		} else {
			// suche ASS am letzten Verbindungspunkt
			foundsa = netz.findeAssAnkommend(getLast().letzterVP());
		}

		return foundsa;
	}

	/**
	 * Bestimmt die Location, auf die sich die Route zubewegt.
	 * 
	 * @param netz
	 *            Importnetz
	 * 
	 * @return Location
	 */
	public Location bestimmeLocationNach(ImportNetz netz) {
		StrassenAbschnitt foundsa = bestimmeEndabschnitt(netz);
		if (foundsa != null) {
			return foundsa.getLocationNach();
		}

		// endet nicht mit ASS -> geht nach UNBEKANNT
		return Location.LOCATION_UNBEKANNT;
	}

	/**
	 * Bestimmt die Location, von der sich die Route wegbewegt.
	 * 
	 * @return Location
	 */
	public Location bestimmeLocationVon() {
		if (get(0).istASS()) {
			return get(0).getLocationNach();
		}

		// beginnt nicht mit ASS -> kommt von UNBEKANNT
		return Location.LOCATION_UNBEKANNT;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.util.AbstractList#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}

		if (getClass() != obj.getClass()) {
			return false;
		}

		if (!super.equals(obj)) {
			return false;
		}

		// final Route other = (Route) obj;
		// if (this.size() != other.size()) {
		// return false;
		// }
		//
		// for (int i = 0; i < this.size(); i++) {
		// if (!this.get(i).equals(other.get(i))) {
		// return false;
		// }
		// }

		return true;
	}

	/**
	 * Berechnet die L&auml;nge der Route.
	 * 
	 * @return L&auml;nge als Summe aller L&auml;ngen der in der Route
	 *         enthaltenen Stra&szlig;enabschnitte
	 */
	double getLaenge() {
		double laenge = 0;

		for (StrassenAbschnitt sal : this) {
			laenge += sal.getLaenge();
		}

		return laenge;
	}

	/**
	 * Gibt das Polygon der Route zur&uuml;ck.
	 * 
	 * @return Poygon
	 */
	public WGS84Polygon getPolygon() {
		List<WGS84Punkt> punkte = new ArrayList<WGS84Punkt>();
		// doppelte Koordinaten entfernen
		// LinkedHashSet<WGS84Punkt> punkte = new LinkedHashSet<WGS84Punkt>();

		for (StrassenAbschnitt abschnitt : this) {
			punkte.addAll(abschnitt.getPolygon().getKoordinaten());
		}

		return new WGS84Polygon(punkte);
		// return new WGS84Polygon(new ArrayList<WGS84Punkt>(punkte));
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.util.AbstractCollection#toString()
	 */
	@Override
	public String toString() {
		StrassenAbschnitt sas = this.getFirst();
		StrassenAbschnitt sae = this.getLast();
		String ses = sas.get(0).getID();
		String see = sae.get(sae.size() - 1).getID();
		return "(R: " + ses.substring(ses.indexOf('.') + 1) + //$NON-NLS-1$
				"->" + see.substring(see.indexOf('.') + 1) + ")"; //$NON-NLS-1$//$NON-NLS-2$
	}
}
