/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuelleKonfiguration;
import de.bsvrz.kex.bwsin.konfig.aktuell.SINAktuellesObjekt;
import de.bsvrz.kex.bwsin.vew.SINProperties;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Factory zur Generierung von PID's und Namen f&uuml;r die
 * Stra&szlig;enelemente. Es wird die Eindeutigkeit der PID sichergestellt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SegmentFactory.java 10247 2008-07-04 08:18:31Z gieseler $
 * 
 */
public class SegmentFactory {
	/**
	 * Erzeugt eine PID (ohne zus&auml;tzliche Fahrbeziehung) f&uuml;r ein
	 * &auml;u&szlig;eres Stra&szlig;ensegment. Die PID wird entsprechend Afo
	 * gebildet.
	 * 
	 * @param locationvon
	 *            Location, von der sich das Stra&szlig;enelement wegbewegt
	 * @param locationnach
	 *            Location, auf die sich das Stra&szlig;enelement zubewegt
	 * 
	 * @return PID
	 */
	public static String bildePIDASS(Location locationvon, Location locationnach) {
		return "ass." + bildePIDLocation(locationvon, locationnach);
	}

	/**
	 * Erzeugt eine PID (ohne zus&auml;tzliche Fahrbeziehung) f&uuml;r ein
	 * inneres Stra&szlig;enteilsegment. Die PID wird entsprechend Afo gebildet.
	 * 
	 * @param locationvon
	 *            Location, von der sich das Stra&szlig;enelement wegbewegt
	 * @param locationnach
	 *            Location, auf die sich das Stra&szlig;enelement zubewegt
	 * 
	 * @return PID
	 */
	@SuppressWarnings("nls")
	public static String bildePIDISS(Location locationvon, Location locationnach) {
		return "iss." + bildePIDLocation(locationvon, locationnach);
	}

	/**
	 * Erzeugt den Location-Teil der PID f&uuml;r innere und &auml;u&szlig;ere
	 * Stra&szlig;enelemente.
	 * 
	 * @param locationvon
	 *            Location, von der sich das Stra&azlig;ensegment wegbewegt
	 * @param locationnach
	 *            Location, auf die sich das Stra&azlig;ensegment zubewegt
	 * 
	 * @return Locationangabe entsprechend DatK.
	 */
	@SuppressWarnings("nls")
	private static String bildePIDLocation(final Location locationvon,
			final Location locationnach) {
		String pidk = locationvon.getLocationRichtung() + "."
				+ locationvon.getLocationCode() + "."
				+ locationnach.getLocationRichtung() + "."
				+ locationnach.getLocationCode();
		return pidk;
	}

	/**
	 * Bildet den Namen eines Stra&szlig;ensegmentes aus PID und optionalem
	 * Zusatztext.
	 * 
	 * @param pid
	 *            PID
	 * @param zusatz
	 *            Zusatztext zum Namen
	 * @return Name des Stra&szlig;ensegmentes
	 */
	public static String bildeSegmentNamen(String pid, String zusatz) {
		String name = pid;
		if (zusatz != null && zusatz.length() > 0) {
			name += " " + zusatz; //$NON-NLS-1$
		}
		return name;
	}

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Bundesland.
	 */
	private String bundesland = "BW"; //$NON-NLS-1$

	/**
	 * Liste der (noch) zu vergleichenden Stra&szlig;enteilsegmente.
	 */
	private List<SINAktuellesObjekt> zutestendeTeilsegmente = null;

	/**
	 * Liste der (noch) zu vergleichenden Stra&szlig;ensegmente.
	 */
	private List<SINAktuellesObjekt> zutestendeSegmente = null;

	/**
	 * neu erzeugte PID.
	 */
	private final HashSet<String> neuePids = new HashSet<String>();

	/**
	 * aktuelle Konfiguration.
	 */
	private SINAktuelleKonfiguration aktuelleKonfiguration = null;

	/**
	 * laufende Nummer Pid Stra&szlig;enteilsegment.
	 */
	private int laufendeNummerTeilSegment = 1;

	/**
	 * Konstruktor.
	 * 
	 * @param aktuelleKonfiguration
	 *            aktuelle Konfiguration
	 */
	public SegmentFactory(final SINAktuelleKonfiguration aktuelleKonfiguration) {
		if (SINProperties.getInstance().containsKey(
				SINProperties.PROP_KEY_BUNDESLAND)) {
			bundesland = SINProperties.getInstance().getProperty(
					SINProperties.PROP_KEY_BUNDESLAND);
		}

		this.aktuelleKonfiguration = aktuelleKonfiguration;

		debug
				.info("Die Segmente der aktuellen Konfiguration werden initialisiert");
		// bilde Listenkopien der aktuellen Objekte
		zutestendeTeilsegmente = new ArrayList<SINAktuellesObjekt>(
				aktuelleKonfiguration.getObjektListe("typ.straenTeilSegment"));
		zutestendeSegmente = new ArrayList<SINAktuellesObjekt>(
				aktuelleKonfiguration.getObjektListe("typ.straenSegment"));
	}

	/**
	 * Bildet die PID f&uuml;r ein Stra&szlig;ensegment.
	 * 
	 * <p>
	 * Zun&auml;chst wird in der aktuellen Konfiguration nach einem identischen
	 * Segment gesucht. Wenn kein passendes existierendes Stra&szlig;ensegment
	 * gefunden wurde, wird aus dem &uuml;bergebenen PID-Anfang eine eindeutige
	 * PID durch Anh&auml;ngen einer laufenden Nummer erzeugt.
	 * </p>
	 * 
	 * @param pidanfang
	 *            zu vergleichende PID (ohne Zusatzinfo)
	 * @param polygon
	 *            zu vergleichendes Polygon
	 * @return eindeutige PID f&uuml;r ein Stra&szlig;ensegment
	 */
	public String bildePIDStrassenSegment(final String pidanfang,
			final WGS84Polygon polygon) {
		SINAktuellesObjekt aktuellesObjekt = findeAktuellesObjekt(
				"typ.straenSegment", pidanfang, polygon);
		String pid = "";

		if (aktuellesObjekt == null) {
			int lfdnr = 0;
			do {
				pid = pidanfang + "." + lfdnr++;
			} while (neuePids.contains(pid)
					|| (aktuelleKonfiguration.getObjekt(pid) != null));
		} else {
			pid = aktuellesObjekt.getPid();
			debug.finer("Das existierende Segment '" + pid
					+ "' wird wiederverwendet");
		}

		neuePids.add(pid);
		return pid;
	}

	/**
	 * Bildet die PID f&uuml;r ein Stra&szlig;enteilsegment.
	 * 
	 * <p>
	 * Zun&auml;chst wird in der aktuellen Konfiguration nach einem identischen
	 * Teilsegment gesucht. Wenn kein passendes existierendes
	 * Stra&szlig;enteilsegment gefunden wurde, wird eine eindeutige PID
	 * erzeugt.
	 * </p>
	 * 
	 * @param polygon
	 *            zu vergleichendes Polygon
	 * @return eindeutige PID f&uuml;r ein Stra&szlig;enteilsegment
	 */
	public String bildePIDStrassenTeilSegment(final WGS84Polygon polygon) {
		SINAktuellesObjekt aktuellesObjekt = findeAktuellesObjekt(
				"typ.straenTeilSegment", "", polygon);
		String pid = "";

		if (aktuellesObjekt == null) {
			do {
				pid = bildePIDSTS(laufendeNummerTeilSegment++);
			} while (neuePids.contains(pid)
					|| (aktuelleKonfiguration.getObjekt(pid) != null));
			debug.finer("Ein neues Teilsegment '" + pid + "' wird angelegt");

		} else {
			pid = aktuellesObjekt.getPid();
			debug.finer("Das existierende Teilsegment '" + pid
					+ "' wird wiederverwendet");
		}

		neuePids.add(pid);
		return pid;
	}

	/**
	 * Erzeugt eine PID f&uuml;r ein Stra&szlig;enteilsegment. Die PID wird
	 * entsprechend Afo gebildet.
	 * 
	 * @param lfdnr
	 *            laufende Nummer
	 * 
	 * @return PID
	 */
	public String bildePIDSTS(final int lfdnr) {
		final String pid = "sts." + String.format("%05d", lfdnr) + "." + bundesland; //$NON-NLS-2$
		return pid;
	}

	/**
	 * Diese Methode sucht ein Objekt der aktuellen Konfiguration, das mit einem
	 * vom Import erzeugten Objekt identisch ist. Ein neues Objekt gilt dabei im
	 * Vergleich zur aktuellen Konfiguration als bereits existierend
	 * (identisch), wenn
	 * <p>- die Regeln zur Bildung der Attribute zur gleichen PID des Objekts
	 * f&uuml;hren und
	 * </p>
	 * <p> - die Koordinaten des Objekts (Punkt oder Polygon) identisch sind.
	 * </p>
	 * <p>
	 * Als identisch werden dabei Koordinaten gewertet, deren Abweichung kleiner
	 * oder gleich einer maximalen Abweichung ist.
	 * </p>
	 * 
	 * @param davtyp
	 *            PID des zu testenden DAV-Typs
	 * @param neuepid
	 *            PID des neuen Objektes
	 * @param polygon
	 *            Polygonzug des neuen Objektes
	 * 
	 * @return das gefundene aktuelle Objekt oder null, wenn kein passendes
	 *         Objekt gefunden wurde
	 */
	private SINAktuellesObjekt findeAktuellesObjekt(String davtyp,
			String neuepid, WGS84Polygon polygon) {
		if (davtyp.equals("typ.straenTeilSegment")) {
			return findeAktuellesTeilsegment(polygon);
		}

		if (davtyp.equals("typ.straenSegment")) {
			return findeAktuellesSegment(neuepid, polygon);
		}

		return null;
	}

	/**
	 * Sucht in der Liste der Stra&szlig;ensegmente der aktuellen Konfiguration
	 * nach einem identischen Segment.
	 * 
	 * @param neuepid
	 *            zu vergleichende PID
	 * @param polygon
	 *            zu vergleichendes Polygon
	 * @return gefundenes aktuelles Objekt oder null, wenn kein Objekt gefunden
	 *         werden konnte
	 */
	private SINAktuellesObjekt findeAktuellesSegment(final String neuepid,
			final WGS84Polygon polygon) {
		double maxabweichung = SINProperties.getInstance()
				.getMaximaleAbweichungGrad();

		SINAktuellesObjekt found = null;

		for (SINAktuellesObjekt so : zutestendeSegmente) {
			// die PID ist hier noch ohne Zusatzinfos
			if (so.getPid().startsWith(neuepid)) {
				if (polygon.istIdentisch(so.getPolygon(), maxabweichung)) {
					found = so;
					break;
				}
			}
		}

		if (found != null) {
			zutestendeSegmente.remove(found);
		}

		return found;
	}

	/**
	 * Findet in der aktuellen Konfiguration ein Teilsegment mit identischem
	 * Polygonzug.
	 * 
	 * @param polygon
	 *            das zu vergleichende Polygon
	 * @return das gefundene Objekt wenn gefunden, oder null
	 */
	private SINAktuellesObjekt findeAktuellesTeilsegment(WGS84Polygon polygon) {
		double maxabweichung = SINProperties.getInstance()
				.getMaximaleAbweichungGrad();

		SINAktuellesObjekt found = null;

		for (SINAktuellesObjekt so : zutestendeTeilsegmente) {
			if (polygon.istIdentisch(so.getPolygon(), maxabweichung)) {
				found = so;
				break;
			}
		}

		if (found != null) {
			zutestendeTeilsegmente.remove(found);
		}

		return found;
	}
}
