/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;
import java.util.List;

import org.geotools.feature.Feature;

/**
 * Repr&auml;sentiert einen Aufteilungspunkt f&uuml;r ein Stra&szlig;enelement.
 * <p>
 * Ein Teilungspunkt ist durch eine Menge von Eigenschaften (Features)
 * gekennzeichnet, die bei ihm beginnen und/oder enden.
 * </p>
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: Teilungspunkt.java 7933 2008-04-03 17:41:03Z gieseler $
 * 
 */
public class Teilungspunkt {

	/**
	 * Die Feature-ID des zugeh&ouml;rigen Abschnittes/Astes.
	 */
	private String aoaFeatureID = null;

	/**
	 * neue Eigenschaften, die bei diesem Punkt beginnen.
	 */
	private final List<Feature> featuresAnfang = new ArrayList<Feature>();

	/**
	 * Eigenschaften, die bei diesem Punkt enden.
	 */
	private final List<Feature> featuresEnde = new ArrayList<Feature>();

	/**
	 * Stationierungswert.
	 */
	private int stationierung = 0;

	/**
	 * interner Referenzierungsz&auml;hler.
	 */
	private int used = 0;

	/**
	 * Erzeugt einen neuen Teilungspunkt.
	 * 
	 * @param sta
	 *            StreckenTeilAbschnitt, auf dem der Punkt liegt
	 * @param beginn
	 *            Feature, das an diesem Punkt beginnt
	 * @param ende
	 *            Feature, das an diesem Punkt endet
	 */
	public Teilungspunkt(final StreckenTeilAbschnitt sta, final Feature beginn,
			final Feature ende) {
		aoaFeatureID = sta.getAOAId();

		if (beginn != null) {
			featuresAnfang.add(beginn);
			stationierung = sta.stationierungAnfang;
		}

		if (ende != null) {
			featuresEnde.add(ende);
			stationierung = sta.stationierungEnde;
		}
	}

	// public Teilungspunkt(WGS84Punkt anfang, Feature f) {
	// this(anfang, null, f);
	// }
	//
	// public Teilungspunkt(WGS84Punkt anfang, WGS84Punkt ende, Feature f) {
	// _anfang = anfang;
	// _ende = ende;
	// }

	/**
	 * F&uuml;gt an diesem Punkt beginnende Features hinzu.
	 * 
	 * @param features
	 *            Liste von Features
	 * @param overwrite
	 *            Flag, ob existierende Features &uuml;berschrieben werden
	 *            sollen
	 */
	public void addFeaturesBeginn(final List<Feature> features,
			final boolean overwrite) {
		for (Feature f : features) {
			if (featuresAnfang.contains(f) && overwrite) {
				featuresAnfang.remove(features);
				featuresAnfang.add(f);
			} else {
				featuresAnfang.add(f);
			}
		}
	}

	/**
	 * F&uuml;gt an diesem Punkt endende Features hinzu.
	 * 
	 * @param features
	 *            Liste von Features
	 */
	public void addFeaturesEnde(List<Feature> features) {
		this.featuresEnde.addAll(features);
	}

	// /**
	// * Berechnet die Koordinaten des Punktes auf der Basis AoA.
	// *
	// * @param okstradb
	// * @return Koordinaten
	// */
	// public WGS84Punkt berechneKoordinaten(Okstra okstradb) {
	// Feature aoaf = okstradb.getFeature(OkstraFeatures.ABSCHNITT,
	// aoaFeatureID);
	//
	// if (aoaf == null) {
	// aoaf = okstradb.getFeature(OkstraFeatures.AST, aoaFeatureID);
	// }
	//
	// if (aoaf != null) {
	// // double korrfak = okstradb.berechneLaengenkorrektur(aoaf);
	// // WGS84Polygon apoly =
	// // Okstra.getFeatureKoordinatenAlsWGS84Polygon(aoaf);
	// // return apoly.bildPunkt(_stationierung*korrfak);
	// }
	//
	// throw new IllegalArgumentException(
	// "Koordinaten Teilungspunkt knnen nicht bestimmt werden");
	// }

	// public WGS84Punkt get_anfang() {
	// return punkt;
	// }

	/**
	 * Gibt die Feature-ID des Abschnittes/Astes zur&uuml;ck, auf dem der Punkt
	 * liegt.
	 * 
	 * @return Feature-ID des zugeh&ouml;rigen Abschnittes/Astes.
	 */
	public String getAoa() {
		return aoaFeatureID;
	}

	/**
	 * Gibt die an diesem Punkt beginnenden Features zur&uuml;ck.
	 * 
	 * @return Liste von Features
	 */
	public List<Feature> getFeaturesBeginn() {
		return featuresAnfang;
	}

	/**
	 * Gibt die an diesem Punkt endenden Features zur&uuml;ck.
	 * 
	 * @return Liste von Features
	 */
	public List<Feature> getFeaturesEnde() {
		return featuresEnde;
	}

	/**
	 * Liefert eine textuelle Beschreibung des Grundes der Aufteilung.
	 * 
	 * @return Text
	 */
	@SuppressWarnings("nls")
	public String getGrund() {
		String grund = "AoA: " + aoaFeatureID + ", Station: " + stationierung
				+ ": ";
		for (Feature f : featuresAnfang) {
			grund += "Beginn " + f.getFeatureType().getTypeName() + " ";
		}

		for (Feature f : featuresEnde) {
			grund += "Ende " + f.getFeatureType().getTypeName() + " ";
		}

		return grund;
	}

	/**
	 * Gibt den Stationierungswert zur&uuml;ck.
	 * 
	 * @return Stationierung
	 */
	public int getStationierung() {
		return stationierung;
	}

	/**
	 * Gibt den Wert des Referenzz&auml;hlers zur&uuml;ck.
	 * 
	 * @return Referenzz&auml;hler
	 */
	public int getUsed() {
		return used;
	}

	// public void set_anfang(WGS84Punkt _anfang) {
	// this.punkt = _anfang;
	// }

	/**
	 * Inkrementiert den Referenzz&auml;hler.
	 */
	public void setUsed() {
		used++;
	}

	/**
	 * Erstellt eine Liste von zu vererbenden Features.
	 * 
	 * @return Liste
	 */
	public List<Feature> vererbe() {
		List<Feature> erbe = new ArrayList<Feature>();

		for (Feature fb : featuresAnfang) {
			boolean match = false;
			for (Feature fe : featuresEnde) {
				if (fb.getFeatureType().equals(fe.getFeatureType())) {
					match = true;
					break;
				}
			}
			if (!match) {
				erbe.add(fb);
			}
		}

		return erbe;
	}
}
