/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;

import org.geotools.feature.Feature;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Rep&auml;sentiert einen Verbindungspunkt von Stra&szlig;enelementen in der
 * internen Netzrepr&auml;sentation.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: Verbindungspunkt.java 7539 2008-03-18 11:54:53Z gieseler $
 * 
 */
public class Verbindungspunkt {

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Flag, ob der Verbindungspunkt benutzt wird.
	 */
	private boolean used = false;

	/**
	 * das zugeh&ouml;rige Feature.
	 */
	private final Feature feature;

	/**
	 * Liste der vom Verbindungspunkt abgehenden Stra&szlig;enelemente.
	 */
	private final ArrayList<StrassenElement> strassenElementeVon = new ArrayList<StrassenElement>();

	/**
	 * Liste der am Verbindungspunkt ankommenden Stra&szlig;enelemente.
	 */
	private final ArrayList<StrassenElement> strassenElementeNach = new ArrayList<StrassenElement>();

	/**
	 * Liste der vom Verbindungspunkt abgehenden Stra&szlig;enabschnitte.
	 */
	private ArrayList<StrassenAbschnitt> strassenAbschnitteVon = new ArrayList<StrassenAbschnitt>();

	/**
	 * Liste der am Verbindungspunkt ankommenden Stra&szlig;enabschnitte.
	 */
	private ArrayList<StrassenAbschnitt> strassenAbschnitteNach = new ArrayList<StrassenAbschnitt>();

	/**
	 * Konstruktur auf der Basis eines Verbindungspunkt-Features der OKSTRA-
	 * Datenbasis.
	 * 
	 * @param feature
	 *            Feature vom Typ 'okstra:Verbindungspunkt'
	 */
	public Verbindungspunkt(Feature feature) {
		this.feature = feature;
	}

	/**
	 * F&uuml;gt ein Stra&szlig;enelement zur Liste der ankommenden
	 * Stra&szlig;enelemente des Verbindungspunktes hinzu.
	 * 
	 * @param strassenelement
	 *            {@link StrassenElement}
	 */
	public void addNach(StrassenElement strassenelement) {
		strassenElementeNach.add(strassenelement);
	}

	/**
	 * F&uuml;gt einen Stra&szlig;enabschnitten der Liste der ankommenden
	 * Stra&szlig;enabschnitte hinzu.
	 * 
	 * @param abschnitt
	 *            {@link StrassenAbschnitt}
	 */
	public void addStrassenAbschnittNach(StrassenAbschnitt abschnitt) {
		strassenAbschnitteNach.add(abschnitt);
	}

	/**
	 * F&uuml;gt einen Stra&szlig;enabschnitten der Liste der abgehenden
	 * Stra&szlig;enabschnitte hinzu.
	 * 
	 * @param abschnitt
	 *            {@link StrassenAbschnitt}
	 */
	public void addStrassenAbschnittVon(StrassenAbschnitt abschnitt) {
		strassenAbschnitteVon.add(abschnitt);
	}

	/**
	 * F&uuml;gt ein Stra&szlig;enelement zur Liste der abgehenden
	 * Stra&szlig;enelemente des Verbindungspunktes hinzu.
	 * 
	 * @param strassenelement
	 *            {@link StrassenElement}
	 */
	public void addVon(StrassenElement strassenelement) {
		strassenElementeVon.add(strassenelement);
	}

	/**
	 * Sucht ein nachfolgendes Stra&szlig;enelement.
	 * 
	 * @param element
	 *            Stra&szlig;enelement
	 * @return nachfolgendes Stra&szlig;enelement oder null
	 */
	public StrassenElement findeNachfolger(StrassenElement element) {
		StrassenElement foundse = null;
		for (StrassenElement se : strassenElementeVon) {
			if (se.ueberpruefeLocationen(element)) {
				if (foundse != null) {
					debug.error("mehrere potentielle Nachfolger");
				}

				foundse = se;
			}
		}

		return foundse;
	}

	/**
	 * Sucht ein Vorg&auml;nger-Stra&szlig;enelement.
	 * 
	 * @param element
	 *            Stra&szlig;enelement
	 * @return Vorg&auml;nger-Stra&szlig;enelement oder null
	 */
	public StrassenElement findeVorgaenger(StrassenElement element) {
		StrassenElement foundse = null;
		for (StrassenElement se : strassenElementeNach) {
			if (se.ueberpruefeLocationen(element)) {
				if (foundse != null) {
					debug.error("mehrere potentielle Vorgaenger");
				}

				foundse = se;
			}
		}

		return foundse;
	}

	/**
	 * Gibt die Liste der abgehenden Stra&szlig;enelemente zur&uuml;ck.
	 * 
	 * @return Liste von {@link StrassenElement}
	 */
	public ArrayList<StrassenElement> getElementeAbgehend() {
		return strassenElementeVon;
	}

	/**
	 * Gibt die Liste der ankommenden Stra&szlig;enelemente zur&uuml;ck.
	 * 
	 * @return Liste von {@link StrassenElement}
	 */
	public ArrayList<StrassenElement> getElementeAnkommend() {
		return strassenElementeNach;
	}

	/**
	 * Gibt das mit dem Verbindungspunkt assoziierte Feature zur&uuml;ck.
	 * 
	 * @return Liste von {@link StrassenElement}
	 */
	public Feature getFeature() {
		return feature;
	}

	/**
	 * Gibt die Feature-ID des assozierten Features zur&uuml;ck.
	 * 
	 * @return Feature-ID
	 */
	public String getID() {
		return (feature.getID());
	}

	/**
	 * Gibt die Liste der ankommenden Stra&szlig;enabschnitte zur&uuml;ck.
	 * 
	 * @return Liste von {@link StrassenAbschnitt}
	 */
	public ArrayList<StrassenAbschnitt> getStrassenAbschnitteNach() {
		return strassenAbschnitteNach;
	}

	/**
	 * Gibt die Liste der abgehenden Stra&szlig;enabschnitte zur&uuml;ck.
	 * 
	 * @return Liste von {@link StrassenAbschnitt}
	 */
	public ArrayList<StrassenAbschnitt> getStrassenAbschnitteVon() {
		return strassenAbschnitteVon;
	}

	/**
	 * Test, ob der Verbindungspunkt bereits benutzt ist.
	 * 
	 * @return Wert des internen Flags.
	 */
	public boolean isUsed() {
		return used;
	}

	/**
	 * Weist dem Verbindungspunkt eine Liste von ankommenden
	 * Stra&szlig;enabschnitten zu.
	 * 
	 * @param abschnitteNach
	 *            Liste von {@link StrassenAbschnitt}
	 */
	public void setStrassenAbschnitteNach(
			ArrayList<StrassenAbschnitt> abschnitteNach) {
		strassenAbschnitteNach = abschnitteNach;
	}

	/**
	 * Weist dem Verbindungspunkt eine Liste von abgehenden
	 * Stra&szlig;enabschnitten zu.
	 * 
	 * @param abschnitteVon
	 *            Liste von {@link StrassenAbschnitt}
	 */
	public void setStrassenAbschnitteVon(
			ArrayList<StrassenAbschnitt> abschnitteVon) {
		strassenAbschnitteVon = abschnitteVon;
	}

	/**
	 * Setzt das Flag, dass den Verbindungspunkt als benutzt markiert.
	 * 
	 * @param used
	 *            true/false
	 */
	public void setUsed(boolean used) {
		this.used = used;
	}
}
