/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.geotools.feature.Feature;

import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraFeatures;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraVerboteneFahrbeziehung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Repr&auml;sentiert die verbotenen Fahrbeziehungen.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: VerboteneFahrbeziehungen.java 10402 2008-07-10 09:07:00Z gieseler $
 */
public class VerboteneFahrbeziehungen {

	/**
	 * Eine Fahrbeziehung.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * @version $Id: VerboteneFahrbeziehungen.java 10402 2008-07-10 09:07:00Z gieseler $
	 * 
	 */
	class Fahrbeziehung {
		/**
		 * Fahrbeziehung von-Stra&szlig;enelement.
		 */
		private String vonStrassenelement = null;
		/**
		 * Fahrbeziehung &uuml;ber-Stra&szlig;enelement.
		 */
		private String ueberStrassenelement = null;
		/**
		 * Fahrbeziehung nach-Stra&szlig;enelement.
		 */
		private String nachStrassenelement = null;

		/**
		 * Erzeugt eine neue verbotene Fahrbeziehung.
		 * 
		 * @param von
		 *            von-Stra&szlig;enelement
		 * @param ueber
		 *            &uuml;ber-Stra&szlig;enelement
		 * @param nach
		 *            nach-Stra&szlig;enelement
		 */
		public Fahrbeziehung(String von, String ueber, String nach) {
			vonStrassenelement = von;
			ueberStrassenelement = ueber;
			nachStrassenelement = nach;
		}

		/**
		 * Gibt das NACH-Stra&szlig;enelement der Fahrbeziehung zur&uuml;ck.
		 * 
		 * @return NACH-Stra&szlig;enelement der Fahrbeziehung
		 */
		protected String getNachStrassenelement() {
			return nachStrassenelement;
		}

		/**
		 * Gibt das &Uuml;BER-Stra&szlig;enelement der Fahrbeziehung
		 * zur&uuml;ck.
		 * 
		 * @return &Uuml;BER-Stra&szlig;enelement der Fahrbeziehung
		 */
		protected String getUeberStrassenelement() {
			return ueberStrassenelement;
		}

		/**
		 * Gibt das VON-Stra&szlig;enelement der Fahrbeziehung zur&uuml;ck.
		 * 
		 * @return VON-Stra&szlig;enelement der Fahrbeziehung
		 */
		protected String getVonStrassenelement() {
			return vonStrassenelement;
		}

		/**
		 * Test, ob eine Liste von Stra&szlig;enelementen eine verbotene
		 * Fahrbeziehung ist.
		 * 
		 * @param elemente
		 *            Liste von Stra&szlig;enelementen
		 * @return true oder false
		 */
		private boolean teste(List<StrassenElement> elemente) {
			if (elemente.size() < 2) {
				return false;
			}

			if (ueberStrassenelement == null) {
				if (elemente.get(0).getID().equals(vonStrassenelement)
						&& elemente.get(1).getID().equals(nachStrassenelement)) {
					return true;
				}
			}

			return false;
		}
	}

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * HashMap der verbotenen Fahrbeziehungen mit Schl&uuml;ssel
	 * 'von-Strassenelement'.
	 */
	private final Map<String, List<Fahrbeziehung>> fbMap = new HashMap<String, List<Fahrbeziehung>>();

	/**
	 * HashMap der aus den verbotenen Fahrbeziehungen resultierenden neuen
	 * Anf&auml;nge von Stra&szlig;ensegmenten.
	 */
	private final HashSet<String> neueAnfaenge = new HashSet<String>();

	/**
	 * F&uuml;gt eine neue Fahrbeziehung ein.
	 * 
	 * @param fb
	 *            Fahrbeziehung
	 */
	private void addFahrbeziehung(Fahrbeziehung fb) {
		List<Fahrbeziehung> fblist = fbMap.get(fb.getVonStrassenelement());

		if (fblist == null) {
			fblist = new ArrayList<Fahrbeziehung>();
			fblist.add(fb);
			fbMap.put(fb.getVonStrassenelement(), fblist);
		} else {
			fblist.add(fb);
		}
	}

	/**
	 * Berechnet der erlaubten Anfang einer Route unter Ber&uuml;cksichtigung
	 * der verbotenen Fahrbeziehungen.
	 * 
	 * @param route
	 *            Liste von Stra&szlig;enabschnitten
	 * @return Anfang der Route bis zur verbotenen Fahrbeziehung
	 */
	public LinkedList<StrassenAbschnitt> bildeErlaubtenAnfang(
			LinkedList<StrassenAbschnitt> route) {
		LinkedList<StrassenAbschnitt> ergebnisroute = route;

		// bilde einen Abschnitt ber die gesamte Route
		StrassenAbschnitt testabschnitt = route.get(0).clone();

		for (int i = 1; i < route.size(); i++) {
			StrassenAbschnitt t = route.get(i);
			for (StrassenElement e : t) {
				testabschnitt.anfuegenEnde(e);
			}
		}

		if (enthaelt(testabschnitt)) {
			// erzeuge neue Route bis zum Beginn der verbotenen Fahrbeziehung
			StrassenElement ee = getErlaubterTeil(testabschnitt);
			ergebnisroute = new LinkedList<StrassenAbschnitt>();
			boolean found = false;
			for (int i = 0; i < route.size() && !found; i++) {
				StrassenAbschnitt ra = route.get(i);
				if (ra.contains(ee)) {
					// der Strassenabschnitt innerhalb der Route, der das
					// letzte erlaubte Strassenelement enthlt
					break;
				}

				ergebnisroute.add(ra);
			}
		}

		return ergebnisroute;
	}

	/**
	 * Berechnet der erlaubten Anfang einer Route unter Ber&uuml;cksichtigung
	 * der verbotenen Fahrbeziehungen.
	 * 
	 * @param route
	 *            Liste von Stra&szlig;enabschnitten
	 * @param importNetz
	 *            Importnetz
	 * @return Liste von Stra&szlig;enabschnitten bis zur ersten enthaltenen
	 *         verbotenen Fahrbeziehung
	 */
	public Route bildeErlaubtenAnfang(Route route, ImportNetz importNetz) {
		Route ergebnisroute = route;

		// bilde einen Abschnitt ber die gesamte Route
		StrassenAbschnitt testabschnitt = route.get(0).clone();

		for (int i = 1; i < route.size(); i++) {
			StrassenAbschnitt t = route.get(i);
			for (StrassenElement e : t) {
				testabschnitt.anfuegenEnde(e);
			}
		}

		if (enthaelt(testabschnitt)) {
			// erzeuge neue Route bis zum Beginn der verbotenen Fahrbeziehung
			StrassenElement ee = getErlaubterTeil(testabschnitt);
			ergebnisroute = new Route();
			boolean found = false;
			for (int i = 0; i < route.size() && !found; i++) {
				StrassenAbschnitt ra = route.get(i);
				if (ra.contains(ee)) {
					// der Strassenabschnitt innerhalb der Route, der das
					// letzte erlaubte Strassenelement enthlt

					// erlaube diese Route nur, wenn der letzte Abschnitt
					// max. einen Nachfolger hat (Abfahrten)
					List<StrassenAbschnitt> nachfolger = importNetz
							.getNachfolger(ra);
					if ((nachfolger == null) || (nachfolger.size() <= 1)) {
						ergebnisroute.add(ra);
					} else {
						ergebnisroute.clear();
					}

					break;
				}

				ergebnisroute.add(ra);
			}
		}

		return ergebnisroute;
	}

	/**
	 * Test, ob eine Liste von Abschnitten eine verbotene Fahrbeziehung
	 * enth&auml;lt.
	 * 
	 * @param route
	 *            Liste von Abschnitten
	 * @return true, wenn ein Abschnitt der Liste eine verbotene Fahrbeziehung
	 *         enth&auml;lt, sonst false
	 */
	public boolean enthaelt(LinkedList<StrassenAbschnitt> route) {

		// bilde einen Abschnitt ber alles
		StrassenAbschnitt testabschnitt = route.get(0).clone();

		for (int i = 1; i < route.size(); i++) {
			StrassenAbschnitt t = route.get(i);
			for (StrassenElement e : t) {
				testabschnitt.anfuegenEnde(e);
			}
		}

		return enthaelt(testabschnitt);
	}

	/**
	 * Test, ob ein Abschnitt eine verbotene Fahrbeziehung enth&auml;lt.
	 * 
	 * @param abschnitt
	 *            zu testender Abschnitt
	 * @return true, wenn der Abschnitt eine verbotene Fahrbeziehung
	 *         enth&auml;lt, sonst false
	 */
	public boolean enthaelt(final StrassenAbschnitt abschnitt) {
		for (int i = 0; i < abschnitt.size(); i++) {
			StrassenElement element = abschnitt.get(i);
			if (fbMap.containsKey(element.getID())) {
				for (Fahrbeziehung fb : fbMap.get(element.getID())) {
					if (fb.teste(abschnitt.subList(i, abschnitt.size()))) {
						return true;
					}
				}
			}
		}

		return false;
	}

	/**
	 * Gibt das Stra&szlig;enelement des Abschnittes zur&uuml;ck, bis zu dem die
	 * Fahrbeziehung erlaubt ist.
	 * 
	 * @param abschnitt
	 *            {@link StrassenAbschnitt}
	 * @return letztes erlaubtes {@link StrassenElement} des Abschnittes
	 */
	public StrassenElement getErlaubterTeil(StrassenAbschnitt abschnitt) {
		for (int i = 0; i < abschnitt.size(); i++) {
			StrassenElement element = abschnitt.get(i);
			if (fbMap.containsKey(element.getID())) {
				for (Fahrbeziehung fb : fbMap.get(element.getID())) {
					if (fb.teste(abschnitt.subList(i, abschnitt.size()))) {
						return element;
					}
				}
			}
		}

		return null;
	}

	/**
	 * Initialisierungsfunktion.
	 * 
	 * @param okstradb
	 *            die Okstra-Datenbasis
	 * @return Anzahl der erzeugten Teilungspunkte
	 */
	public int initialisiere(final Okstra okstradb) {
		int added = 0;
		Map<String, Feature> fmap;
		Feature feature;

		DEBUG.info("Die verbotenen Fahrbeziehungen werden initialisiert."); //$NON-NLS-1$

		try {
			fmap = okstradb
					.getFeatureMap(OkstraFeatures.VERBOTENE_FAHRBEZIEHUNG
							.getFeatureOkstraName());

			for (Map.Entry<String, Feature> e : fmap.entrySet()) {
				feature = e.getValue();

				String von = Okstra.getRefFeatureId((String) feature
						.getAttribute(OkstraVerboteneFahrbeziehung.VON));

				String ueber = (String) feature
						.getAttribute(OkstraVerboteneFahrbeziehung.UEBER);
				if (ueber != null) {
					ueber = Okstra.getRefFeatureId(ueber);
				}

				String nach = (String) feature
						.getAttribute(OkstraVerboteneFahrbeziehung.NACH);
				if (nach != null) {
					nach = Okstra.getRefFeatureId(nach);
				}

				addFahrbeziehung(new Fahrbeziehung(von, ueber, nach));

				neueAnfaenge.add(nach);
			}
		} catch (final IllegalArgumentException e1) {
			DEBUG.error(e1.getMessage());
		} catch (Exception e) {
			DEBUG.error(e.getMessage());
		}

		DEBUG
				.info("" + fbMap.size() + " verbotene Fahrbeziehunge(n) initialisiert"); //$NON-NLS-1$//$NON-NLS-2$

		return added;
	}

	/**
	 * Test, ob ein Stra&szlig;enabschnitt ein neuer Anfang aus einer verbotenen
	 * Fahrbeziehung ist.
	 * 
	 * @param abschnitt
	 *            Stra&szlig;enabschnitt
	 * @return true, wenn der Stra&szlig;enabschnitt ein neuer Anfang aus einer
	 *         verbotenen Fahrbeziehung ist, sonst false
	 */
	public boolean istNeuerAnfangAusVerbotenerFahrbeziehung(
			StrassenAbschnitt abschnitt) {
		return neueAnfaenge.contains(abschnitt.getFirst().getID());
	}

	/**
	 * Test, ob durch das Anf&uuml;gen eine Stra&szlig;enabschnittes an eine
	 * Route eine verbotene Fahrbeziehungen entsteht.
	 * 
	 * @param froute
	 *            Route
	 * @param abschnitt
	 *            Stra&szlig;enabschnitt
	 * 
	 * @return true, wenn die Route durch das Anf&uuml;gen des
	 *         Stra&szlig;enabschnittes eine verbotenen Fahrbeziehung wird, onst
	 *         false
	 */
	public boolean istVerboten(Route froute, StrassenAbschnitt abschnitt) {
		Route testroute = new Route(froute);
		testroute.add(abschnitt);
		return enthaelt(testroute);
	}

}
