/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.lcl;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import de.bsvrz.kex.bwsin.SINMsg;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.csv.csvleser.CsvLeser;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert das Modul LCL.
 * 
 * Es werden Zugriffsfunktionen auf die Daten der Location-Code-Liste
 * bereitgestellt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: LCL.java 7802 2008-03-28 18:23:55Z gieseler $
 */

public final class LCL implements SINLCLInterface {

	/**
	 * Konstanten fr den Zugriff auf die Spalten der LCL.
	 */
	/** Vorg&auml;nger-Location. */
	public static final String SPALTE_VORGAENGER = "NEGATIVE_OFFSET"; //$NON-NLS-1$
	/** Nachfolger-Location. */
	public static final String SPALTE_NACHFOLGER = "POSITIVE_OFFSET";
	/** Linienreferenz. */
	public static final String LINEAR_REFERENZ = "LINEAR_REFERENCE";
	/** Stra&szlig;ennummer. */
	public static final String ROADNUMBER = "ROADNUMBER";
	/** Erster Name. */
	public static final String FIRST_NAME = "FIRST_NAME";
	/** Zweiter Name. */
	public static final String SECOND_NAME = "SECOND_NAME";
	/** Stra&szlig;enname. */
	public static final String ROADNAME = "ROADNAME";
	/** Intersection-Code. */
	public static final String INTERSECTIONCODE = "INTERSECTIONCODE";
	/** Netzknoten-Nr. */
	public static final String NETZKNOTEN_NR = "NETZKNOTEN_NR";
	/** Location-Typ. */
	public static final String LOCATION_TYP = "TYPE";
	/** Location-Subtyp. */
	public static final String LOCATION_SUBTYP = "SUBTYPE";
	/** Zeigt an, ob die Location in positiver Richtung definiert ist. */
	public static final String PRESENT_POSITIVE = "PRESENT_POSITIVE";
	/** Zeigt an, ob die Location in negativer Richtung definiert ist. */
	public static final String PRESENT_NEGATIVE = "PRESENT_NEGATIVE";

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Instanz.
	 */
	private static LCL instance = new LCL();

	/**
	 * Liefert die Instanz der LCL zur&uuml;ck.
	 * 
	 * @return LCL
	 */
	public static LCL getInstance() {
		return instance;
	}

	/**
	 * Initialisiert die interne Liste aus einer LCL-Datei.
	 * 
	 * @param dateiName
	 *            der Name der LCL-Datei (CSV)
	 * @throws IllegalArgumentException
	 *             bei ung&uuml;ltigem Dateinamen
	 * @throws FileNotFoundException
	 *             wenn die Datei nicht ge&ouml;ffnet werden konnte
	 */
	public static void initialisiere(final String dateiName)
			throws FileNotFoundException {
		instance = new LCL();
		instance.init(dateiName);
	}

	/**
	 * Die LCL als Map.
	 */
	private final Map<Object, String[]> lclMap = new TreeMap<Object, String[]>();

	/**
	 * Der Header der LCL.
	 */
	private List<?> lclHeader = null;

	/**
	 * Der in der LCL-Datei benutzte Element-Trenner.
	 */
	private final char lclDelimiter = ';';

	/**
	 * Privater Konstruktor.
	 */
	private LCL() {
		// never used
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.lcl.SINLCLInterface#getLocationCodes(java.lang.String,
	 *      java.lang.String)
	 */
	public List<String> getLocationCodes(final String spaltenname,
			final String spaltenwert) {
		final Collection<String[]> map = lclMap.values();
		final List<String> rlist = new LinkedList<String>();

		for (final String[] im : map) {
			final int spind = lclHeader.indexOf(spaltenname);
			if (im[spind].equals(spaltenwert)) {
				rlist.add(im[0]);
			}
		}
		return rlist;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.lcl.SINLCLInterface#getLocationCodeWert(java.lang.String,
	 *      java.lang.String)
	 */
	public String getLocationCodeWert(final String locationcode,
			final String spaltenname) {
		if (lclMap.containsKey(locationcode)) {
			return lclMap.get(locationcode)[lclHeader.indexOf(spaltenname)];
		}
		return null;
	}

	/**
	 * Gibt die Anzahl der Location-Codes der LCL zur&uuml;ck.
	 * 
	 * @return Anzahl Location-Codes
	 */
	public int getsize() {
		return lclMap.size();
	}

	/**
	 * Initialisiert die Klasse aus einer LCL-Datei.
	 * 
	 * @param dateiName
	 *            Name der LCL-Datei.
	 * @throws FileNotFoundException
	 *             wenn die Datei nicht ge&ouml;ffnet werden konnte
	 */
	private void init(final String dateiName) throws FileNotFoundException {
		if ((dateiName == null) || (dateiName.length() == 0)) {
			throw new IllegalArgumentException(
					"Der Dateiname der LCL darf nicht null oder leer sein.");
		}

		if (!new File(dateiName).exists()) {
			throw new FileNotFoundException("Die angegebene Datei '"
					+ dateiName + "' existiert nicht.");
		}

		LogTools.log(DEBUG, SINMsg.InfoInitLCL, dateiName);

		final Reader inputStream = new FileReader(dateiName);

		final CsvLeser cvsreader = new CsvLeser(inputStream, lclDelimiter);
		lclHeader = cvsreader.getErsteZeile();

		Map<String, String> lmap = null;

		while (cvsreader.hasNochMehrZeilen()) {
			lmap = cvsreader.getNaechsteZeile();
			// aus Effizienzgrnden wird die LCL nicht als Hash-Map von
			// Hash-Maps
			// sondern als Hash-Map von String-Arrays angelegt
			// Hash-Key ist der Location-Code
			final String[] cols = new String[lmap.size()];
			for (int i = 0; i < lmap.size(); i++) {
				cols[i] = lmap.get(lclHeader.get(i));
			}
			lclMap.put(cols[0], cols);
		}
	}
}
