/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra;

import java.util.HashMap;
import java.util.Map;

import org.geotools.feature.FeatureType;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import de.bsvrz.kex.bwsin.konfig.okstra.parser.SINFeatureHandler;

/**
 * Feature(typ) der OKSTRA-Datenbasis.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: OkstraFeature.java 10401 2008-07-10 09:06:04Z gieseler $
 * 
 */
public abstract class OkstraFeature {

	/**
	 * Name des Features in der OKSTRA-Datenbasis.
	 */
	private final String okstraname;

	/**
	 * interner Name.
	 */
	private final String name;

	/**
	 * Prefix Feature-Name.
	 */
	private final String prefix;

	/**
	 * Flag, ob der eigene Parser benutzt werden soll.
	 */
	private boolean eigenerParser = false;

	/**
	 * Das zugeh&ouml;rige XML-Dokument.
	 */
	private Document doc = null;

	/**
	 * Tabelle der Features.
	 */
	Map<String, SINFeatureHandler> fliste = new HashMap<String, SINFeatureHandler>();

	/**
	 * Tabelle der Knoten des Dokumentes.
	 */
	Map<String, Node> nodemap = new HashMap<String, Node>();

	/**
	 * Konstruktor.
	 * 
	 * @param feature
	 *            Feature-Name
	 * @param name
	 *            Name
	 * @param prefix
	 *            Prefix Feature-Name
	 * @param eigenerparser
	 *            eigener Parser soll benutzt werden
	 */
	public OkstraFeature(String feature, String name, String prefix,
			Boolean eigenerparser) {
		okstraname = feature;
		this.name = name;
		this.prefix = prefix;
		this.eigenerParser = eigenerparser.booleanValue();
	}

	/**
	 * Gibt das zugeh&ouml;rige XML-Dokument zur&uuml;ck.
	 * 
	 * @return Document
	 */
	public Document getDocument() {
		return doc;
	}

	/**
	 * Gibt den Namen des Features zur&uuml;ck.
	 * 
	 * @return Name
	 */
	public String getFeatureName() {
		return name;
	}

	/**
	 * Gibt den Namen des Features in der OKSTRA-Datenbasis zur&uuml;ck.
	 * 
	 * @return Name in der OKSTRA-Datenbasis
	 */
	public String getFeatureOkstraName() {
		return okstraname;
	}

	/**
	 * Gibt die Features zur&uuml;ck.
	 * 
	 * @return Tabelle der Features
	 */
	public Map<String, SINFeatureHandler> getFeatures() {
		return this.fliste;
	}

	/**
	 * Gibt den Knoten mit einer bestimmten ID zur&uuml;ck.
	 * 
	 * @param id
	 *            Knoten-ID
	 * @return Knoten oder null
	 */
	public Node getNode(String id) {
		return nodemap.get(id);
	}

	/**
	 * Gibt den Prefix des Features zur&uuml;ck.
	 * 
	 * @return Prefix des Features
	 */
	public String getPrefix() {
		return prefix;
	}

	/**
	 * Gibt die Information zur&uuml;ck, ob das Feature mit dem eigenen Parser
	 * behandelt werden soll.
	 * 
	 * @return true oder false
	 */
	protected boolean isEigenerParser() {
		return eigenerParser;
	}

	/**
	 * Test, ob das Feature von einem bestimmten Typ ist.
	 * 
	 * @param ft
	 *            Feature-Typ
	 * @return treu, wenn das Feature vom angegebenen Typ ist, sonst false
	 */
	public boolean istFeatureTyp(final FeatureType ft) {
		String ftname = ft.getTypeName(); // ist ohne Prefix
		String oname;
		if (okstraname.contains(":")) {
			oname = okstraname.substring(okstraname.indexOf(':') + 1);
		} else {
			oname = okstraname;
		}

		return oname.equals(ftname);
	}

	/**
	 * Setzt das XML-Dokument.
	 * 
	 * @param newdoc
	 *            Document
	 */
	public void setDocument(Document newdoc) {
		this.doc = newdoc;
	}

	/**
	 * Setzt die Feature-Tabelle.
	 * 
	 * @param newfliste
	 *            Feature-Tabelle
	 */
	public void setFeatures(final Map<String, SINFeatureHandler> newfliste) {
		this.fliste = newfliste;
	}

	/**
	 * Setzt die Knotentabelle.
	 * 
	 * @param newnodemap
	 *            Knotentabelle
	 */
	public void setMap(Map<String, Node> newnodemap) {
		this.nodemap = newnodemap;
	}
}
